//SerialPort.h

/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPFserial is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/
/*
	Current Author: Marty Kraimer, Thomas Coleman
	Original Author: Jim Kowalkowski (for Hideos)
	Date: 09JUL98
*/
 
#ifndef SerialPortH
#define SerialPortH

#include <epicsEvent.h>
#include "serialStatus.h"

enum requestType {requestNone,requestRead,requestWrite};
enum byteHandlerRC {byteHandlerOK,byteHandlerEndRead,byteHandlerError};

typedef byteHandlerRC (*BYTE_HANDLER)
    (void* parm,unsigned char byte);
/*
    byte handler is called by input routine. It must return one of:
    byteHandlerOK  normal return
    byteHandlerEndRead end of input message has been detected
    byteHandlerError - An error occured - end operation

*/

typedef void (*PEEK_HANDLER)
    (void* parm,bool isReceive, unsigned char byte);
// If registered PEEK_HANDLER is called for every byte transmitted or received

class SerialPort
{
public:
    SerialPort();
    int registerName(const char *portName);
    static SerialPort *find(const char *portName);
    static SerialPort *bind(const char *portName,
        void* byteHandlerParm,BYTE_HANDLER byteHandler);
    void release();
    static bool setPeekHandler(
	const char *portName,void* peekHandlerParm,PEEK_HANDLER peekHandler);
    static void freePeekHandler(const char *portName);
    virtual serialStatus write(
	unsigned char *data, int nbytes, double timeout) = 0;
    virtual serialStatus read(double timeout) =0;
    virtual void flush();
    virtual bool config( int baud, int stopBits, int bitsPerChar,
	char parity, char flowControl);
    virtual void getConfig( int *baud, int *stopBits, int *bitsPerChar,
	char *parity, char *flowControl);
protected:
    BYTE_HANDLER byteHandler;
    void* byteHandlerParm;
    PEEK_HANDLER peekHandler;
    void* peekHandlerParm;
private:
    bool isBound;
    const char *portName;
    bool firstFlush;
    bool firstConfig;
    bool firstGetConfig;
};

#endif // SerialPortH
